<?php

namespace Drupal\sign_widget\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;
use Drupal\file\Entity\File;
use Drupal\image\Plugin\Field\FieldWidget\ImageWidget;

/**
 * Plugin implementation of the 'image_image' widget.
 *
 * @FieldWidget(
 *   id = "sign_widget",
 *   label = @Translation("Sign"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class SignWidget extends ImageWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'dotSize' => 1,
        'minWidth' => 0.5,
        'maxWidth' => 2.5,
        'backgroundColor' => '',
        'penColor' => '#000000',
        'velocityFilterWeight' => 0.7,
        'signTool' => FALSE,
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = [];

    $elements['dotSize'] = [
      '#type' => 'number',
      '#title' => t('Dot Size'),
      '#default_value' => $this->getSetting('dotSize'),
      '#required' => TRUE,
      '#description' => t('Radius of a single dot.'),
    ];

    $elements['minWidth'] = [
      '#type' => 'number',
      '#step' => '.1',
      '#title' => t('Minimum Width'),
      '#default_value' => $this->getSetting('minWidth'),
      '#required' => TRUE,
      '#description' => t('Minimum width of a line'),
    ];

    $elements['maxWidth'] = [
      '#type' => 'number',
      '#step' => '.1',
      '#title' => t('Maximum Width'),
      '#default_value' => $this->getSetting('maxWidth'),
      '#element_validate' => ['element_validate_number'],
      '#required' => TRUE,
      '#description' => t('Maximum width of a line.'),
    ];

    $elements['backgroundColor'] = [
      '#type' => 'color',
      '#title' => t('Background Color'),
      '#default_value' => $this->getSetting('backgroundColor'),
      '#description' => t('Color used to clear the background.'),
    ];

    $elements['penColor'] = [
      '#type' => 'color',
      '#title' => t('Pen Color'),
      '#default_value' => $this->getSetting('penColor'),
      '#required' => TRUE,
      '#description' => t('Color used to draw the lines.'),
    ];

    $elements['velocityFilterWeight'] = [
      '#type' => 'number',
      '#step' => '.1',
      '#title' => t('Velocity Filter Weight'),
      '#default_value' => $this->getSetting('velocityFilterWeight'),
      '#required' => TRUE,
      '#description' => t('Weight used to modify new velocity based on the previous velocity.'),
    ];
    $elements['signTool'] = [
      '#type' => 'checkbox',
      '#title' => t('Toolbox'),
      '#default_value' => $this->getSetting('signTool'),
      '#description' => t('Show toolbox to modify dot size and color pen.'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if (!empty($this->getSetting('dotSize')) && !empty($this->getSetting('minWidth')) && !empty($this->getSetting('maxWidth'))) {
      $summary[] = t('Size :  (%maxWidth x %minWidth)%size', [
        '%size' => $this->getSetting('dotSize'),
        '%minWidth' => $this->getSetting('minWidth'),
        '%maxWidth' => $this->getSetting('maxWidth'),
      ]);
    }
    if (!empty($this->getSetting('backgroundColor'))) {
      $summary[] = t('Background Color : %backgroundColor', ['%backgroundColor' => $this->getSetting('backgroundColor')]);
    }
    if (!empty($this->getSetting('penColor'))) {
      $summary[] = t('Pen Color : %penColor', ['%penColor' => $this->getSetting('penColor')]);
    }
    if (!empty($this->getSetting('velocityFilterWeight'))) {
      $summary[] = t('Velocity Filter Weight : %velocityFilterWeight', ['%velocityFilterWeight' => $this->getSetting('velocityFilterWeight')]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {

    $values = $items->getValue();
    $settings = $this->fieldDefinition->getSetting("default_image");
    $width = !empty($settings['width']) ? $settings['width'] : 288;
    $height = !empty($settings['height']) ? $settings['height'] : 180;
    if (empty($element['#upload_validators'])) {
      $element['#upload_validators'] = FALSE;
    }
    $field_name = $this->fieldDefinition->getName();
    $id = Html::getId(implode('-', $element["#field_parents"]) . '-' . $field_name);
    $fid = $img = '';
    if (!empty($values[$delta])) {
      $fid = $values[$delta]['target_id'];
      $file = File::load($fid);
    }
    if (empty($file) && !empty($settings["uuid"])) {
      $file = \Drupal::service('entity.repository')
        ->loadEntityByUuid('file', $settings["uuid"]);
    }
    if (!empty($file)) {
      $imgSrc = $file->createFileUrl();
      $img = '<img class="background" src="' . $imgSrc . '"  width="' . $width . '" height="' . $height . '"/>';
    }
    $element['#signature_size'] = [
      'height' => $height,
      'width' => $width,
    ];
    if ($this->settings['backgroundColor'] == '#ffffff' || !empty($img)) {
      $this->settings['backgroundColor'] = 'rgba(255, 255, 255, 0)';
    }
    $sign_pad = [
      '#type' => 'html_tag',
      '#tag' => 'canvas',
      '#attributes' => [
        'id' => $id,
        'width' => $width,
        'height' => $height,
        'class' => ["signature-pad"],
      ],
      '#prefix' => '<div class="signature_panel">' . $img,
    ];
    if (!empty($this->settings['signTool'])) {
      $btn_group = '<div class="clearfix toolbox"><div class="btn-group btn-group-sm" role="group">
  <input data-id="' . $id . '" type="range" min="0" max="10" data-type="dotSize" step="1" class="form-control form-range"/>
  <input data-id="' . $id . '" type="color" data-type="penColor" class="form-control form-control-color"/>
  <button data-id="' . $id . '" type="button" class="clear btn btn-outline-secondary">' . t('Clear') . '</button>
  </div></div>';
      $sign_pad['#suffix'] = $btn_group;
    }

    $element['#attached']['drupalSettings']['sign_pad'][$id] = $this->settings;
    foreach ($this->settings as $attribute => $settingValue) {
      $sign_pad['#attributes']['data-' . $attribute] = $settingValue;
    }
    $element += [
      '#type' => 'hidden',
      '#required' => FALSE,
      '#attributes' => [
        'id' => $id . '-sign',
        'class' => ['signature-storage'],
      ],
    ];
    //in case modify
    if (!empty($fid)) {
      $element['#default_value'] = $fid;
    }
    $element['#attached']['library'][] = 'sign_widget/sign_widget';

    $element['#prefix'] = '<div class="esign_container">';
    $element['#suffix'] = \Drupal::service('renderer')
      ->renderPlain($sign_pad);
    $file_extensions = $this->fieldDefinition->getSetting("file_extensions");
    return [
      'value' => $element,
      '#upload_validators' => [
        'file_validate_extensions' => [$file_extensions],
      ],
      '#required' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $file_directory = $this->fieldDefinition->getSetting("file_directory");
    $uri = \Drupal::config('system.file')->get('default_scheme') . '://';
    if (!empty($file_directory)) {
      $uri .= rtrim($file_directory, '/') . '/';
      $path = \Drupal::service('file_system')->realpath($uri);
      $dir = \Drupal::service('file_system')
        ->prepareDirectory($path, FileSystemInterface::CREATE_DIRECTORY);
    }
    foreach ($values as &$value) {
      if (!empty($value['value'])) {
        if (is_numeric($value['value'])) {
          $file = File::load($value['value']);
        }
        else {
          $encoded_image = explode(",", $value['value'])[1];
          $decoded_image = base64_decode($encoded_image);
          $filename = date('ymd') . '_' . rand(1000, 9999) . '.png';
          //Saves a file to the specified destination and creates a database entry.
          $file = file_save_data($decoded_image, $uri . $filename, FileSystemInterface::EXISTS_REPLACE);
        }
        [$width, $height] = getimagesize($file->getFileUri());
        $value = [
          'target_id' => $file->id(),
          'width' => $width,
          'height' => $height,
        ];
      }
      else {
        unset($value['value']);
      }
    }
    return $values;
  }

}
